@@ -1,6 +1,6 @@
 (*
  * XmlRpc Light, a small XmlRpc library based on Xml Light and Ocamlnet
- * Copyright (C) 2007 Dave Benjamin (dave@ramenlabs.com)
+ * Copyright (C) 2007-2009 Dave Benjamin (dave@ramenlabs.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
-let version = "0.6"
+let version = "0.6.1"
 
 exception Error of (int * string)
 
@@ -143,11 +143,24 @@ let rec value_of_xml_element
           `Struct
             (safe_map
                (function
+                  | Xml.Element ("member", [],
+                                 [Xml.Element ("name", [], []);
+                                  Xml.Element ("value", [], [])]) ->
+                      (* Empty value is assumed to be an empty string. *)
+                      ("", `String "")
                   | Xml.Element ("member", [],
                                  [Xml.Element ("name", [], [Xml.PCData name]);
                                   Xml.Element ("value", [], [])]) ->
                       (* Empty value is assumed to be an empty string. *)
                       (name, `String "")
+                  | Xml.Element ("member", [],
+                                 [Xml.Element ("name", [], []);
+                                  Xml.Element ("value", [], [value])]) ->
+                      ("",
+                       value_of_xml_element
+                         ~base64_decoder
+                         ~datetime_decoder
+                         value)
                   | Xml.Element ("member", [],
                                  [Xml.Element ("name", [], [Xml.PCData name]);
                                   Xml.Element ("value", [], [value])]) ->
@@ -522,6 +535,17 @@ let default_error_handler e =
 let quiet_error_handler e =
   raise e
 
+let serve_message ?(error_handler=default_error_handler) f message =
+  try
+    match message with
+      | MethodCall (name, params) ->
+          (try MethodResponse (f name params) with
+             | Error _ as e -> raise e
+             | e -> error_handler e)
+      | _ -> invalid_xmlrpc ()
+  with Error (code, string) ->
+    Fault (code, string)
+
 let serve
     ?(base64_encoder=fun s -> XmlRpcBase64.str_encode s)
     ?(base64_decoder=fun s -> XmlRpcBase64.str_decode s)
@@ -529,27 +553,23 @@ let serve
     ?(datetime_decoder=XmlRpcDateTime.of_string)
     ?(error_handler=default_error_handler)
     f s =
+  fix_dotted_tags s;
   try
     begin
       try
-        begin
-          fix_dotted_tags s;
-          match (message_of_xml_element
-                   ~base64_decoder
-                   ~datetime_decoder
-                   (Xml.parse_string s))
-          with
-            | MethodCall (name, params) ->
-                Xml.to_string_fmt
-                  (xml_element_of_message
-                     ~base64_encoder
-                     ~datetime_encoder
-                     (try MethodResponse (f name params) with
-                        | Error _ as e -> raise e
-                        | e -> error_handler e))
-            | _ -> invalid_xmlrpc ()
-        end
-      with Xml.Error _ -> invalid_xml ()
+        let message =
+          message_of_xml_element
+            ~base64_decoder
+            ~datetime_decoder
+            (Xml.parse_string s) in
+        let response =
+          xml_element_of_message
+            ~base64_encoder
+            ~datetime_encoder
+            (serve_message ~error_handler f message) in
+        Xml.to_string_fmt response
+      with Xml.Error _ ->
+        invalid_xml ()
     end
   with Error (code, string) ->
     Xml.to_string_fmt (xml_element_of_message (Fault (code, string)))
